// Demonstrate Color and Interpolation

#include "Angel.h"

//Define some geometry and colours here
GLfloat points[] = 
{
	-1, 0, 0,
	 1, 0, 0,
	 0, 1, 0
};

GLfloat colors[] = 
{
	 1, 0, 0,
	 0, 1, 0,
	 0, 0, 1
};


//Vertex array objects
GLuint vao;

//Shader Programs
GLuint program1;

//Color Uniform for Shader
GLint shaderColor;

//----------------------------------------------------------------------------

// OpenGL initialization
void init()
{	
	//Set up shaders and VAOs
	program1 = InitShader( "vShaderL4-1.glsl", "fShaderL4-1.glsl" );

	glUseProgram( program1 );
	glGenVertexArrays( 1, &vao );
	glBindVertexArray( vao );

    //Pack points and colors into one buffer
	GLuint buffer;
	glGenBuffers( 1, &buffer );
	glBindBuffer( GL_ARRAY_BUFFER, buffer );
	glBufferData( GL_ARRAY_BUFFER, sizeof(points)+sizeof(colors), NULL, GL_STATIC_DRAW );
	glBufferSubData( GL_ARRAY_BUFFER, 0, sizeof(points), points );
	glBufferSubData( GL_ARRAY_BUFFER, sizeof(points), sizeof(colors), colors );

    //Connect point data at beginning of buffer to vPosition attribute
    GLuint vPosition = glGetAttribLocation( program1, "vPosition" );
	glEnableVertexAttribArray( vPosition );
    glVertexAttribPointer( vPosition, 3, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(0) );
	
    //Connect color data at end of  buffer to vColor attribute
    GLuint vColor = glGetAttribLocation( program1, "vColor" );
	glEnableVertexAttribArray( vColor );
	glVertexAttribPointer( vColor, 3, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(sizeof(points)) );
}

//----------------------------------------------------------------------------

void display( void )
{
	//Clear the drawing buffer
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	//Draw triangle
	glDrawArrays(GL_TRIANGLES, 0, 3);

	//Show what was drawn
	glutSwapBuffers();
}

//----------------------------------------------------------------------------

void reshape (int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
}


void keyboard( unsigned char key, int x, int y )
{
	switch( key ) {
	case 033:  // Escape key
	case 'q': case 'Q':
		exit( EXIT_SUCCESS );
		break;
	}
}

//----------------------------------------------------------------------------


int main( int argc, char **argv )
{
	glutInit( &argc, argv );
	glutInitWindowSize( 512, 512 );

#ifdef __APPLE__
	glutInitDisplayMode( GLUT_3_2_CORE_PROFILE | GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
#else
	// If you are using freeglut, the next two lines will check if 
	// the code is truly 3.2. Otherwise, comment them out
	glutInitContextVersion( 3, 2 );
	glutInitContextProfile( GLUT_CORE_PROFILE );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
	
#endif

	glutCreateWindow( "Lab 4 - Demo 1" );
	glewExperimental = GL_TRUE;
	glewInit();

	init();
	glutDisplayFunc( display );
	glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutMainLoop();
	return 0;
}
